package io.github.beradq.adybubbles

import org.bukkit.command.CommandSender
import org.bukkit.entity.Player
import taboolib.common.platform.command.CommandBody
import taboolib.common.platform.command.CommandHeader
import taboolib.common.platform.command.subCommand
import io.github.beradq.adybubbles.AdyBubbles.entityFinder
import taboolib.common.platform.ProxyCommandSender
import taboolib.common.platform.function.adaptCommandSender
import taboolib.common.platform.function.adaptPlayer
import taboolib.module.chat.component

private fun clearMessageScreen(player: ProxyCommandSender) {
    for (i in 1..40) player.sendMessage(" ")
}

fun sendEditMessage(player: ProxyCommandSender, uuid: String, title: String) {
    clearMessageScreen(player)
    val bubbles = TraitBubblesChat.getBubbles(uuid) ?: Bubbles(mutableListOf(), ChatPopupMode.LOOP)
    player.sendMessage("当前正在编辑: $title 的弹出泡泡")
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage("弹出模式(点击切换):")
    when (bubbles.mode) {
        ChatPopupMode.LOOP ->
            "[\\[循环\\]](b;u) [\\[随机\\]](cmd=/bubbles-chat editdata $uuid mode random) [\\[单次\\]](cmd=/bubbles-chat editdata $uuid mode once)"
                .component().sendTo(player)

        ChatPopupMode.RANDOM ->
            "[\\[循环\\]](cmd=/bubbles-chat editdata $uuid mode loop) [\\[随机\\]](b;u) [\\[单次\\]](cmd=/bubbles-chat editdata $uuid mode once)"
                .component().sendTo(player)

        ChatPopupMode.ONCE ->
            "[\\[循环\\]](cmd=/bubbles-chat editdata $uuid mode loop) [\\[随机\\]](cmd=/bubbles-chat editdata $uuid mode random) [\\[单次\\]](b;u)"
                .component().sendTo(player)
    }
    player.sendMessage(" ")
    "[\\[弹出内容\\](点击编辑)](cmd=/bubbles-chat editdata $uuid items)".component().sendTo(player)
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage("请展开聊天栏编辑")
    player.sendMessage(" ")
    player.sendMessage(" ")
    player.sendMessage(" ")
}

@CommandHeader("bubbles-chat")
@Suppress("unused")
object BubblesChatCommand {

    @CommandBody
    @Suppress("unused")
    val play = subCommand {
        dynamic("UUID") {
            suggestion<CommandSender>(uncheck = true) { _, _ ->
                entityFinder.getEntities().map { it.uniqueId }
            }
            execute<CommandSender> { _, context, _ ->
                val uuid = context["UUID"]
                TraitBubblesChat.startOnce(uuid)
            }
        }
    }

    @CommandBody
    @Suppress("unused")
    val editdata = subCommand {
        dynamic("UUID") {
            suggestion<CommandSender>(uncheck = true) { _, _ ->
                entityFinder.getEntities().map { it.uniqueId }
            }
            dynamic("FIELD") {
                suggestion<CommandSender>(uncheck = false) { _, _ ->
                    listOf("items", "mode")
                }
                execute<CommandSender> { sender, context, _ ->
                    val field = context["FIELD"]
                    if (field != "items") {
                        sender.sendMessage("缺少参数")
                        return@execute
                    }
                    val uuid = context["UUID"]
                    val npc = entityFinder.getEntityFromUniqueId(uuid) ?: throw Exception("Entity not found")
                    TraitBubblesChat.edit(sender as Player, npc)
                    sendEditMessage(adaptCommandSender(sender), uuid, npc.id)
                }
                dynamic("VALUE") {
                    execute<CommandSender> { sender, context, _ ->
                        val field = context["FIELD"]
                        val value = context["VALUE"]
                        val uuid = context["UUID"]
                        val npc = entityFinder.getEntityFromUniqueId(uuid) ?: throw Exception("Entity not found")
                        when (field) {
                            "mode" -> {
                                val mode = ChatPopupMode.fromString(value)
                                TraitBubblesChat.setMode(uuid, mode)
                                sendEditMessage(adaptCommandSender(sender), uuid, npc.id)
                            }

                            else -> {
                                sender.sendMessage("Unknown field: $field")
                            }
                        }
                    }
                }
            }
        }
    }

    @CommandBody
    @Suppress("unused")
    val edit = subCommand {
        dynamic("NPCID") {
            suggestion<CommandSender>(uncheck = true) { _, _ ->
                entityFinder.getEntities().map { it.id }
            }
            execute<CommandSender> { sender, context, _ ->
                val npcs = entityFinder.getEntitiesFromId(context["NPCID"])
                if (npcs.size == 1) {
                    val uuid = npcs[0].uniqueId
                    sendEditMessage(adaptPlayer(sender as Player), uuid, npcs[0].id)
                } else {
                    sender.sendMessage("找到了${npcs.size}个NPC:")
                    for (npc in npcs) {
                        "[${npc.uniqueId}](cmd=/bubbles-chat edit2 ${npc.uniqueId})".component().sendTo(
                            adaptCommandSender(sender)
                        )
                    }
                }
            }
        }
    }

    @CommandBody
    @Suppress("unused")
    val edit2 = subCommand {
        dynamic("UUID") {
            suggestion<CommandSender>(uncheck = true) { _, _ ->
                entityFinder.getEntities().map { it.uniqueId }
            }
            execute<CommandSender> { sender, context, _ ->
                val uuid = context["UUID"]
                val npc = entityFinder.getEntityFromUniqueId(uuid)!!
                sendEditMessage(adaptPlayer(sender as Player), uuid, npc.id)
            }
        }
    }
}