package io.github.beradq.adybubbles

import ink.ptms.adyeshach.core.entity.EntityInstance
import ink.ptms.adyeshach.core.event.AdyeshachEntityVisibleEvent
import ink.ptms.adyeshach.impl.entity.trait.Trait
import ink.ptms.adyeshach.impl.util.Inputs.inputBook
import taboolib.common.LifeCycle
import org.bukkit.entity.Player
import taboolib.common.platform.Awake
import taboolib.common.platform.event.EventPriority
import taboolib.common.platform.event.SubscribeEvent
import taboolib.common.platform.function.submit
import taboolib.common.platform.service.PlatformExecutor
import taboolib.module.chat.uncolored
import java.util.concurrent.CompletableFuture

object TraitBubblesChat : Trait() {
    private val bubbles = mutableMapOf<String, Bubbles>()
    private val visibleEntity = mutableListOf<String>()
    private var handle: PlatformExecutor.PlatformTask? = null

    fun getBubbles(uuid: String): Bubbles? {
        val conf = data.getConfigurationSection(uuid) ?: return null
        val items = conf.getStringList("items")
        val mode = conf.getString("mode")?.let { ChatPopupMode.fromString(it) } ?: ChatPopupMode.LOOP
        return Bubbles(items.toMutableList(), mode)
    }

    @Suppress("unused")
    fun setBubbles(uuid: String, bubbles: Bubbles) {
        data[uuid] = mapOf(
            "items" to bubbles.items,
            "mode" to bubbles.mode.toString()
        )
        uuid pipe this::update
    }

    override fun edit(player: Player, entityInstance: EntityInstance): CompletableFuture<Void> {
        val future = CompletableFuture<Void>()
        val uuid = entityInstance.uniqueId
        val content = bubbles.getOrPut(uuid) { Bubbles(mutableListOf()) }.items
        player.inputBook(content) {
            (future::complete.bind(null))()
            if (it.all { s -> s.isBlank() }) {
                data[uuid] = null
                uuid also bubbles::remove also BubblesBundle::clear
            } else {
                if (!data.contains(uuid)) data[uuid] = mapOf("items" to it.uncolored())
                else data.getConfigurationSection(uuid)!!["items"] = it.uncolored()
            }
            uuid pipe this::update
        }
        return future
    }

    fun setMode(uuid: String, mode: ChatPopupMode) {
        if (!data.contains(uuid)) data[uuid] = mapOf("mode" to mode.toString())
        else data.getConfigurationSection(uuid)!!["mode"] = mode.toString()
        uuid pipe this::update
    }

    private fun update(uuid: String) {
        if (!data.contains(uuid)) return
        val innerData = data.getConfigurationSection(uuid)!!
        val items = if (innerData.contains("items")) innerData.getStringList("items") else emptyList()
        val mode = ChatPopupMode.fromString(innerData.getString("mode") ?: "loop")
        bubbles[uuid] = Bubbles(items.toMutableList(), mode)
    }

    private fun fistUpdate(uuid: String) {
        if (bubbles.containsKey(uuid)) return
        update(uuid)
    }


    @Awake(LifeCycle.ACTIVE)
    private fun startTicking() {
        handle?.cancel()
        handle = submit(period = Config.bubblesConfig.chat.period, async = true) {
            val ve = visibleEntity.toList()
            ve.forEach {
                val bubble = bubbles[it] ?: return@forEach
                val item = when (bubble.mode) {
                    ChatPopupMode.LOOP -> {
                        bubble.next()
                    }

                    ChatPopupMode.RANDOM -> bubble.random()
                    else -> return@forEach
                } ?: return@forEach
                popup(it, item)
            }
        }
    }

    fun startOnce(uuid: String) {
        val bubble = bubbles[uuid] ?: return
        if (bubble.mode != ChatPopupMode.ONCE) return
        bubble.resetState()
        val cloneBubble = bubble.clone()
        submit(period = Config.bubblesConfig.chat.period, async = true) {
            val item = cloneBubble.next() ?: return@submit this.cancel()
            popup(uuid, item)
        }
    }

    fun popup(uuid: String, text: String) {
        if (Config.bubblesConfig.chat.lifetime != null) {
            BubblesBundle.popupTick(uuid, text, Config.bubblesConfig.chat.lifetime!!)
        } else {
            BubblesBundle.popup(uuid, text)
        }
        BubblesBundle.limit(uuid, Config.bubblesConfig.chat.limit)
    }

    @SubscribeEvent(priority = EventPriority.MONITOR, ignoreCancelled = true)
    // 仅在可视时渲染聊天气泡
    private fun onVisible(e: AdyeshachEntityVisibleEvent) {
        if (e.entity.isPublic()) {
            val uuid = e.entity.uniqueId
            fistUpdate(uuid)
            if (e.visible) {
                if (visibleEntity.contains(uuid)) return
                visibleEntity.add(uuid)
            } else {
                uuid also visibleEntity::remove also BubblesBundle::clear
            }
        }
    }

    override fun id(): String {
        return "bubbles-chat"
    }
}
